@@ -350,7 +350,7 @@ on_search_menu_item_activated (GtkMenuItem *menuitem,
                         -1);
 
     if (search_dialog == NULL) {
-        search_dialog = GTK_DIALOG (g_object_new (SEARCH_DIALOG_TYPE, NULL));
+        search_dialog = search_dialog_new ();
     }
 
     search_dialog_set_server (SEARCH_DIALOG (search_dialog), server);
@@ -967,7 +967,7 @@ append_media_server (GUPnPDeviceProxy *proxy,
                                                 "ContainerUpdateIDs",
                                                 G_TYPE_STRING,
                                                 on_container_update_ids,
-                                                NULL);
+                                                info);
                 gupnp_service_proxy_set_subscribed (content_dir, TRUE);
                 g_hash_table_insert (initial_notify, content_dir, content_dir);
 
