@@ -23,6 +23,7 @@
 
 #include <string.h>
 
+#include "entry-completion.h"
 #include "search-dialog.h"
 #include "server-device.h"
 #include "didl-dialog.h"
@@ -66,16 +67,19 @@ struct _SearchDialogPrivate {
 typedef struct _SearchDialogPrivate SearchDialogPrivate;
 G_DEFINE_TYPE_WITH_PRIVATE (SearchDialog, search_dialog, GTK_TYPE_DIALOG)
 
-void
+static void
 search_dialog_on_search_activate (SearchDialog *self, GtkEntry *entry);
 
-gboolean
+static gboolean
 search_dialog_on_listview_button_release (GtkWidget      *widget,
                                           GdkEventButton *event,
                                           gpointer        user_data);
 
-void
-search_dialog_on_didl_popup_activate (SearchDialog *self, gpointer user_data);
+static void
+search_dialog_on_didl_popup_activate (SearchDialog *self, GVariant *parameter, gpointer user_data);
+
+static void
+search_dialog_on_icon_release (SearchDialog *self, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data);
 
 static void
 search_dialog_finalize (GObject *object);
@@ -352,6 +356,13 @@ search_dialog_class_init (SearchDialogClass *klass)
         gtk_widget_class_bind_template_child_private (widget_class,
                                                       SearchDialog,
                                                       search_dialog_treeview);
+        gtk_widget_class_bind_template_callback (widget_class,
+                                                 search_dialog_on_search_activate);
+        gtk_widget_class_bind_template_callback (widget_class,
+                                                 search_dialog_on_listview_button_release);
+        gtk_widget_class_bind_template_callback (widget_class, gtk_widget_hide);
+        gtk_widget_class_bind_template_callback (widget_class, search_dialog_on_icon_release);
+
 
         object_class->finalize = search_dialog_finalize;
         object_class->dispose = search_dialog_dispose;
@@ -361,16 +372,35 @@ static void
 search_dialog_init (SearchDialog *self)
 {
         SearchDialogPrivate *priv = NULL;
-        GtkBuilder *builder = gtk_builder_new ();
 
         gtk_widget_init_template (GTK_WIDGET (self));
         priv = search_dialog_get_instance_private (self);
 
         priv->parser = gupnp_search_criteria_parser_new ();
+        gtk_entry_set_completion (priv->search_dialog_entry,
+                                  entry_completion_new ());
 
-        gtk_builder_add_from_resource (builder, DIALOG_RESOURCE_PATH, NULL);
-        gtk_builder_connect_signals (builder, self);
-        priv->popup_menu = GTK_WIDGET (gtk_builder_get_object (builder, "popup-menu"));
+        GMenu *menu = g_menu_new ();
+        g_menu_insert (menu, 0, _("Show _DIDL…"), "search.show-didl");
+        priv->popup_menu = gtk_menu_new_from_model (G_MENU_MODEL (menu));
+        g_object_unref (menu);
+
+        gtk_menu_attach_to_widget (GTK_MENU (priv->popup_menu),
+                                   GTK_WIDGET (self),
+                                   NULL);
+        GSimpleActionGroup *group = g_simple_action_group_new ();
+        GSimpleAction *action = g_simple_action_new ("show-didl", NULL);
+        g_signal_connect_swapped (G_OBJECT (action),
+                                  "activate",
+                                  G_CALLBACK (search_dialog_on_didl_popup_activate),
+                                  self);
+        g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+
+        gtk_widget_insert_action_group (GTK_WIDGET (self),
+                                        "search",
+                                        G_ACTION_GROUP (group));
+        g_object_unref (group);
+        g_object_unref (action);
 }
 
 static void
@@ -424,11 +454,19 @@ search_dialog_on_search_task_done (gpointer user_data)
 
         g_source_remove (priv->pulse_timer);
         gtk_entry_set_progress_fraction (priv->search_dialog_entry, 0);
-        gtk_widget_set_sensitive (GTK_WIDGET (priv->search_dialog_entry), TRUE);
+        gtk_editable_set_editable (GTK_EDITABLE (priv->search_dialog_entry), TRUE);
+        g_object_set (G_OBJECT (priv->search_dialog_entry),
+                        "primary-icon-name",
+                        "edit-find-symbolic",
+                        "secondary-icon-activatable",
+                        TRUE,
+                        NULL);
 
-        /* Only show visible error if dialog is visible. If it's not visible,
-         * it's likely to be a cancelled error */
+        /* Only show visible error if dialog is visible. */
         if (priv->task->error != NULL &&
+            !g_error_matches (priv->task->error,
+                              G_IO_ERROR,
+                              G_IO_ERROR_CANCELLED) &&
             gtk_widget_is_visible (GTK_WIDGET (self))) {
                 GtkWidget *dialog = NULL;
 
@@ -458,8 +496,12 @@ void
 search_dialog_set_server (SearchDialog *self, AVCPMediaServer *server)
 {
         SearchDialogPrivate *priv = search_dialog_get_instance_private (self);
+        GtkEntryCompletion *completion = gtk_entry_get_completion (priv->search_dialog_entry);
 
         priv->server = server;
+
+        entry_completion_set_search_criteria (ENTRY_COMPLETION (completion),
+                                              av_cp_media_server_get_search_caps (server));
 }
 
 void
@@ -530,8 +572,7 @@ pulse_timer (gpointer user_data)
         return TRUE;
 }
 
-G_MODULE_EXPORT
-void
+static void
 search_dialog_on_search_activate (SearchDialog *self, GtkEntry *entry)
 {
         GError *error = NULL;
@@ -540,8 +581,15 @@ search_dialog_on_search_activate (SearchDialog *self, GtkEntry *entry)
 
         gupnp_search_criteria_parser_parse_text (priv->parser, text, &error);
         if (error == NULL) {
+
+                g_object_set (G_OBJECT (entry),
+                              "primary-icon-name",
+                              "media-playback-stop-symbolic",
+                              "secondary-icon-activatable",
+                              FALSE,
+                              NULL);
                 gtk_list_store_clear (priv->search_dialog_liststore);
-                gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
+                gtk_editable_set_editable (GTK_EDITABLE (entry), FALSE);
                 priv->pulse_timer = g_timeout_add_seconds (1, pulse_timer, self);
 
                 g_clear_pointer (&priv->task, search_task_free);
@@ -618,8 +666,7 @@ do_popup_menu (GtkMenu *menu, GtkWidget *widget, GdkEventButton *event)
 #endif
 }
 
-G_MODULE_EXPORT
-gboolean
+static gboolean
 search_dialog_on_listview_button_release (GtkWidget      *widget,
                                           GdkEventButton *event,
                                           gpointer        user_data)
@@ -724,9 +771,8 @@ search_dialog_on_metadata_ready (GObject *source,
         }
 }
 
-G_MODULE_EXPORT
-void
-search_dialog_on_didl_popup_activate (SearchDialog *self, gpointer user_data)
+static void
+search_dialog_on_didl_popup_activate (SearchDialog *self, GVariant *parameter, gpointer user_data)
 {
         SearchDialogPrivate *priv  = search_dialog_get_instance_private (self);
         GtkTreeView  *treeview = priv->search_dialog_treeview;
@@ -770,3 +816,36 @@ search_dialog_on_didl_popup_activate (SearchDialog *self, gpointer user_data)
                 g_free (id);
         }
 }
+
+GtkDialog *
+search_dialog_new (void)
+{
+        GtkSettings *settings = gtk_settings_get_default ();
+        int use_header;
+
+        g_object_get (G_OBJECT (settings),
+                                "gtk-dialogs-use-header",
+                                &use_header,
+                                NULL);
+
+        return (GtkDialog *) g_object_new (SEARCH_DIALOG_TYPE,
+                                           "use-header-bar",
+                                           use_header == 1 ? TRUE : FALSE,
+                                           NULL);
+}
+
+static void
+search_dialog_on_icon_release (SearchDialog *self, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
+{
+        SearchDialogPrivate *priv  = search_dialog_get_instance_private (self);
+
+        if (icon_pos == GTK_ENTRY_ICON_PRIMARY) {
+                if (priv->task != NULL && priv->task->running) {
+                        search_task_cancel (priv->task);
+                } else {
+                        search_dialog_on_search_activate (self, user_data);
+                }
+        } else {
+                gtk_entry_set_text (GTK_ENTRY (user_data), "");
+        }
+}
