@@ -158,6 +158,8 @@ notify_status_change (gboolean status)
                                       status,
                                       NULL);
         }
+
+        g_list_free (network_lights);
 }
 
 void
@@ -179,6 +181,8 @@ notify_load_level_change (gint load_level)
                                       load_level,
                                       NULL);
         }
+
+        g_list_free (network_lights);
 }
 
 G_MODULE_EXPORT
@@ -409,9 +413,9 @@ static void init_uuid (void)
         char *udn;
 
         uuid = gupnp_get_uuid ();
+        const xmlDoc *xml_doc = gupnp_xml_doc_get_doc (doc);
 
-        uuid_node = xml_util_get_element ((xmlNode *) doc,
-                gupnp_xml_doc_get_doc (doc),
+        uuid_node = xml_util_get_element ((xmlNode *) xml_doc,
                                           "root",
                                           "device",
                                           "UDN",
@@ -430,18 +434,19 @@ static void init_uuid (void)
         g_free (udn);
 }
 
-static void on_service_proxy_action_ret (GUPnPServiceProxy *proxy,
-                                  GUPnPServiceProxyAction *action,
-                                  gpointer user_data)
+static void
+on_service_proxy_action_ret (GObject *object,
+                             GAsyncResult *result,
+                             gpointer user_data)
 {
         GError *error = NULL;
 
-        gupnp_service_proxy_end_action (proxy,
-                                        action,
-                                        &error,
-                                        NULL);
-        if (error) {
-                GUPnPServiceInfo *info = GUPNP_SERVICE_INFO (proxy);
+        gupnp_service_proxy_call_action_finish (GUPNP_SERVICE_PROXY (object),
+                                                result,
+                                                &error);
+
+        if (error != NULL) {
+                GUPnPServiceInfo *info = GUPNP_SERVICE_INFO (object);
 
                 g_warning ("Failed to call action \"%s\" on \"%s\": %s",
                            (char *) user_data,
@@ -464,18 +469,24 @@ set_all_status (gboolean status)
              proxy_node = g_list_next (proxy_node)) {
                 GUPnPServiceProxy *proxy;
                 char *action_name;
+                GUPnPServiceProxyAction *action;
 
                 proxy = GUPNP_SERVICE_PROXY (proxy_node->data);
                 action_name = g_strdup ("SetTarget");
 
-                gupnp_service_proxy_begin_action (proxy,
-                                                  action_name,
-                                                  on_service_proxy_action_ret,
-                                                  action_name,
-                                                  "newTargetValue",
-                                                  G_TYPE_BOOLEAN,
-                                                  status,
-                                                  NULL);
+                action = gupnp_service_proxy_action_new (action_name,
+                                                         "newTargetValue",
+                                                         G_TYPE_BOOLEAN,
+                                                         status,
+                                                         NULL);
+
+                gupnp_service_proxy_call_action_async (
+                        proxy,
+                        action,
+                        NULL,
+                        on_service_proxy_action_ret,
+                        action_name);
+                gupnp_service_proxy_action_unref (action);
         }
 }
 
@@ -489,18 +500,25 @@ set_all_load_level (gint load_level)
              proxy_node = g_list_next (proxy_node)) {
                 GUPnPServiceProxy *proxy;
                 char *action_name;
+                GUPnPServiceProxyAction *action;
 
                 proxy = GUPNP_SERVICE_PROXY (proxy_node->data);
                 action_name = g_strdup ("SetLoadLevelTarget");
 
-                gupnp_service_proxy_begin_action (proxy,
-                                                  action_name,
-                                                  on_service_proxy_action_ret,
-                                                  action_name,
-                                                  "newLoadlevelTarget",
-                                                  G_TYPE_UINT,
-                                                  load_level,
-                                                  NULL);
+                action = gupnp_service_proxy_action_new (action_name,
+                                                         "newLoadlevelTarget",
+                                                         G_TYPE_UINT,
+                                                         load_level,
+                                                         NULL);
+
+                gupnp_service_proxy_call_action_async (
+                        proxy,
+                        action,
+                        NULL,
+                        on_service_proxy_action_ret,
+                        action_name);
+
+                gupnp_service_proxy_action_unref (action);
         }
 }
 
@@ -570,11 +588,15 @@ on_network_light_unavailable (GUPnPControlPoint *control_point,
                 remove_service_from_list (info, &switch_proxies);
         }
 
+        g_clear_object (&info);
+
         info = gupnp_device_info_get_service (GUPNP_DEVICE_INFO (light_proxy),
                                               DIMMING_SERVICE);
         if (info) {
                 remove_service_from_list (info, &dimming_proxies);
         }
+
+        g_clear_object (&info);
 }
 
 static gboolean
@@ -745,7 +767,7 @@ context_equal (GUPnPContext *context1, GUPnPContext *context2)
 }
 
 gboolean
-init_upnp (gchar **interfaces, guint port, gchar *name)
+init_upnp (gchar **interfaces, guint port, gchar *name, gboolean ipv4, gboolean ipv6)
 {
         GUPnPWhiteList *white_list;
 
@@ -761,7 +783,22 @@ init_upnp (gchar **interfaces, guint port, gchar *name)
                 return FALSE;
         }
 
-        context_manager = gupnp_context_manager_create (port);
+        // Default: Both
+        GSocketFamily family = G_SOCKET_FAMILY_INVALID;
+        if (!ipv4 && ipv6) {
+            g_debug ("Option a");
+            family = G_SOCKET_FAMILY_IPV6;
+        } else if (ipv4 && !ipv6) {
+            g_debug ("Option b");
+            family = G_SOCKET_FAMILY_IPV4;
+        } else {
+            g_debug ("Option c");
+            // Neither? Just do nothing and enable both
+        }
+
+        context_manager = gupnp_context_manager_create_full (GSSDP_UDA_VERSION_1_0,
+                                                             family,
+                                                             port);
         g_assert (context_manager != NULL);
 
         if (interfaces != NULL) {
