@@ -150,7 +150,7 @@ main (gint   argc,
                 return -1;
         }
 
-        if (argc < 3) {
+        if (argc < 2) {
                 char *help = NULL;
 
                 help = g_option_context_get_help (context, TRUE, NULL);
@@ -160,6 +160,11 @@ main (gint   argc,
                 return -4;
         }
 
+        if (udn == NULL) {
+                g_print ("Error: Please provide UDN for the target server\n");
+                return -4;
+        }
+
         /* Get the list of files to upload */
         for (i = 1; i < argc; i++) {
                 if (!g_file_test (argv[i],
